@@ -54,8 +54,10 @@ umask 0077
 
 TEMPLATEFILE="$(mktemp -t mellon_create_sp.XXXXXXXXXX)"
 
+dd if=/dev/urandom of=$TEMPLATEFILE.RANDOM bs=256 count=1 2>/dev/null
+
 cat >"$TEMPLATEFILE" <<EOF
-RANDFILE           = /dev/urandom
+RANDFILE           = ${TEMPLATEFILE}.RANDOM
 [req]
 default_bits       = 3072
 default_keyfile    = privkey.pem
@@ -68,13 +70,20 @@ EOF
 
 openssl req -utf8 -batch -config "$TEMPLATEFILE" -new -x509 -days 3652 -nodes -out "$OUTFILE.cert" -keyout "$OUTFILE.key" 2>/dev/null
 
-rm -f "$TEMPLATEFILE"
+rm -f "$TEMPLATEFILE" "${TEMPLATEFILE}.RANDOM"
 
 CERT="$(grep -v '^-----' "$OUTFILE.cert")"
 
 cat >"$OUTFILE.xml" <<EOF
 <EntityDescriptor entityID="$ENTITYID" xmlns="urn:oasis:names:tc:SAML:2.0:metadata" xmlns:ds="http://www.w3.org/2000/09/xmldsig#">
   <SPSSODescriptor protocolSupportEnumeration="urn:oasis:names:tc:SAML:2.0:protocol" AuthnRequestsSigned="true">
+    <KeyDescriptor use="encryption">
+      <ds:KeyInfo xmlns:ds="http://www.w3.org/2000/09/xmldsig#">
+        <ds:X509Data>
+          <ds:X509Certificate>$CERT</ds:X509Certificate>
+        </ds:X509Data>
+      </ds:KeyInfo>
+    </KeyDescriptor>
     <KeyDescriptor use="signing">
       <ds:KeyInfo xmlns:ds="http://www.w3.org/2000/09/xmldsig#">
         <ds:X509Data>
